/* ----------------------------------------
 * Origin  : GFI INFORMATIQUE 
 * Project : CDPP Propagation Tool 
 * ----------------------------------------
 * History 
 * Version : 01/05/2015 : SCA : V2.1
 * ----------------------------------------
 */
package fr.tlse.irap.pt.client.controller.tools.jmap;

import fr.tlse.irap.pt.client.controller.common.ISpaceElementMenu;
import fr.tlse.irap.tool.radial.CMEType;

public interface IJMapMenu extends ISpaceElementMenu {

	public void runChooseMap(String source, String type);
	public void runChooseCatalog(boolean linkcatMode, CMEType cmeType);
	public void runSelectFit();
    public void runClickPoints();
    public void runClickPointsFile();
	public void runRemoveLastPoint();
	public void runFitPoints();
	public void runZoomP1();
	public void runZoomP2();
	public void runResetZoom();
	public void runReset();
	public void runSavePoints();
}
